
import pygame

import data
import utils
from engine import State

FONTNAME = "courier-bold"

class StaticWidget(pygame.Surface):
    """Basic static widget
    Handles drawing and positioning - only useful for inheritance
    """
    def __init__(self, size):
        """StaticWidget constructor
        size: a tuple with 2 integer values
        """
        pygame.Surface.__init__(self, size)
        self.size = size
        self.rect = self.get_rect()

    def set_pos(self, pos):
        """Set widget's position
        """
        self.rect.topleft = pos

    def get_pos(self):
        """Get widget's position
        """
        return self.rect.topleft

    def draw(self, surf):
        """Draw widget on given surface
        """
        surf.blit(self, self.get_pos())

class ImgButton(StaticWidget):
    """Image button
    Click on that image and something happens..
    """
    def __init__(self, img, callback, arg=None):
        """ImgButton constructor
        img: button image
        callback: function to be called when button is clicked
        arg: argument passed with callback
        """
        StaticWidget.__init__(self, img.get_size())
        self.blit(img, (0,0))
        self.callback = callback
        self.arg = arg
        self.hover = False

    def highlight(self):
        """Called when mouse hovers button
        Put your own highlighting stuff here.
        """
        pass

    def unhighlight(self):
        """Called when mouse doesn't hover button anymore
        Put your own unhighlighting stuff here.
        """
        pass

    def update(self):
        """check if button is clicked or (un-)highlighted
        """
        mouseover = self.rect.collidepoint(pygame.mouse.get_pos())
        l_button = pygame.mouse.get_pressed()[0]

        if mouseover and not self.hover:
            self.hover = True
            self.highlight()
        elif not mouseover and self.hover:
            self.hover = False
            self.unhighlight()

        if mouseover and l_button:
            if self.arg is None:
                self.callback()
            else:
                self.callback(self.arg)


class ProgressBar(StaticWidget):
    """Plain progressbar widget
    Useful for healthbars or other progress information
    """
    def __init__(self, size, color, bgcolor):
        """ProgressBar constructor
        size: 2-valued tuple
        color: foreground color (the "filling")
        bgcolor: background color
        """
        StaticWidget.__init__(self, size)
        self.length = size[0]
        self.height = size[1]
        self.color = color
        self.bgcolor = bgcolor
        self.perc = 0

    def change(self, perc):
        """Fill progressbar by perc%
        """
        if perc > 1:
            perc = 1
        elif perc < 0:
            perc = 0
        self.perc = perc
        self.fill(self.bgcolor)
        filling = pygame.Surface((int(self.length*perc),self.height))
        filling.fill(self.color)
        self.blit(filling, (0,0))


class Cursor(pygame.sprite.Sprite):
    """An image that follows the mouse's position
    Useful to replace the default cursor. Don't forget to set
    pygame.mouse.set_visible(False) to hide the default cursor.
    """
    def __init__(self, image, clickpoint=(0,0)):
        """Cursor constructor
        image: cursor image
        clickpoint: (How do I explain that?!)
        Usually that's the topleft or topright pixel of the image.
        If you're using a crosshair as cursor, the clickpoint would probably
        be somewhere near the center.
        """
        pygame.sprite.Sprite.__init__(self)
        self.image = image
        self.base_image = image.copy()
        self.rect = self.image.get_rect()
        sx, sy = image.get_size()
        self.x_off, self.y_off = clickpoint
        self.update()

    def update(self):
        """follow mouse position
        """
        x, y = pygame.mouse.get_pos()
        self.rect.topleft = (x - self.x_off, y - self.y_off)

    def set_pos(self, pos):
        """set position of Mouse and Cursor
        """
        pygame.mouse.set_pos(pos)
        self.rect.topleft = pos

class ClickArea(pygame.Rect):
    """Like a button but without image, just a clickeable area
    """
    def __init__(self, area, callback, arg=None):
        """ClickArea constructor
        area: [x_pos, y_pos, width, height]
        callback: function to be called on clicking the area
        arg: argument passed to callback
        """
        pygame.Rect.__init__(self, area)
        self.callback = callback
        self.arg = arg
        self.hover = False

    def highlight(self):
        """Called when mouse hovers area
        Put your own highlighting stuff here.
        """
        pass

    def unhighlight(self):
        """Called when mouse doesn hover area anymore
        Put your own unhighlighting stuff here.
        """
        pass


    def update(self):
        """check if mouse (un)hovers or clicks area
        """
        mouseover = self.collidepoint(pygame.mouse.get_pos())
        l_button = pygame.mouse.get_pressed()[0]

        if mouseover and not self.hover:
            self.hover = True
            self.highlight()
        elif not mouseover and self.hover:
            self.hover = False
            self.unhighlight()

        if mouseover and l_button:
            if self.arg is None:
                self.callback()
            else:
                self.callback(self.arg)



class TextLine(StaticWidget):
    """Just a StaticWidget with a line of text
    Useful for one-liners like "Game Over."
    """
    def __init__(self, text, fontsize=20, color=(255,255,255), \
        bgcolor=(0,0,0), font=FONTNAME):
        font = data.filepath(font)
        self.font = pygame.font.Font(font, fontsize)
        self.color = color
        self.bgcolor = bgcolor
        img = self.font.render(text, 1, color, bgcolor)
        StaticWidget.__init__(self, img.get_size())
        self.blit(img, (0,0))

    def update(self, text):
        """Change text
        """
        img = self.font.render(text, 1, self.color, self.bgcolor)
        self.fill(self.bgcolor)
        self.blit(img, (0,0))

class TextArea(pygame.Surface):
    """An area of text. Long lines will be wrapped.
    """
    def __init__(self, text, fontsize=15, length=45, font=FONTNAME, \
        color=(255,255,255), bgcolor=(0,0,0)):
        """TextArea constructor
        length: number of characters per line
        """
        font = data.filepath(font)
        font = pygame.font.Font(font, fontsize)
        lineheight = fontsize + 4

        # generate textlines
        words = text.split(" ")
        line = ""
        lines = []
        for w in words:
            if len(line+w) <= length:
                line = line + w + " "
            else:
                lines.append(line)
                line = w + " "
        if line:
            lines.append(line)

        # generate images for textlines
        width = 0
        self.lines = []
        for line in lines:
            img = font.render(line, 1, color, bgcolor)
            self.lines.append(img)
            w = img.get_size()[0]
            if w > width:
                width = w

        # blit imagelines
        size = (width, len(self.lines)*lineheight)
        pygame.Surface.__init__(self, size)
        self.fill(bgcolor)
        y = 0
        for line in self.lines:
            self.blit(line, (0, y))
            y += lineheight


